<?php
/* $Id: referant.cls.php,v 1.17 2010/03/14 21:53:14 sarterm Exp $ */
include_once(dirname(__FILE__)."/config.inc.php");

/*
 * Stocke les informations sur une URL référante.
 * - referant : l'URL complète originale,
 * - hote : le nom d'hôte du serveur référant,
 * - protocole : le protocole correspondant à l'URL (http, ftp, etc...),
 * - baseUrl : l'URL racine du serveur référant,
 * - requete : le chemin d'accés sur le serveur référant,
 * - testMoteur : vaut true s'il a été vérifié si l'URL est un moteur de
 *                recherche,
 * - estMoteur : vaut true si l'URL provient d'un moteur de recherche,
 * - nomMoteur : le nom du moteur de recherche correspondant,
 * - varianteMoteur : la variante du moteur de recherche correspondant (images, blogs...),
 * - requeteMoteur : les termes de recherche qui ont mené à l'URL.
 */
class IBDReferant {
    public $referant = "";
    public $hote = "";
    public $protocole = "";
    public $baseUrl = "";
    public $requete = "";
    public $testMoteur = false;
    public $estMoteur = false;
    public $nomMoteur = "";
    public $varianteMoteur = "";
    public $libelleMoteur = "";
    public $requeteMoteur = "";
    private $init = false;
    private $baseMoteurs = Array();

    /*
     * Constructeur de la classe.
     * Le constructeur prend une URL référante en paramètre et effectue
     * l'analyse de celle-çi.
     *
     * @param ref L'URL référante. Si ce paramètre vaut true, l'URL est
     *            récupérée dans la super globale $_SERVER. Si elle vaut false,
     *            aucune analyse n'est effectuée.
     * @param tester Si à true, l'URL est analysée pour vérifier s'il s'agit
     *               d'un moteur de recherche.
     */
    function IBDReferant($ref = true, $tester = false) {
        if (!$this->init) {
            $this->init();
        }
        if ($ref === false) {
            return 0;
        }
        else if ($ref == true) {
            $ref = $_SERVER["HTTP_REFERER"];
        }
        $now = explode(' ',microtime());
        $start = $now[0] + $now[1];
        $this->referant = $ref;
        $parsed = parse_url($ref);
        $this->hote = isset($parsed["host"]) ? $parsed["host"] : "";
        $this->requete = isset($parsed["query"]) ? $parsed["query"] : "";
        $this->protocole = isset($parsed["scheme"]) ? $parsed["scheme"] : "";
        $this->baseUrl = $this->protocole."://".$this->hote."/";
        if ($tester) {
            $this->testerMoteur();
        }
        $now = explode(' ',microtime());
        $end = $now[0] + $now[1];
        return $end - $start;
    }

    /*
     * Analyse l'URL pour déterminer s'il s'agit ou non d'un moteur de
     * recherche.
     */
    public function testerMoteur($forceTest = false, $forceInit = false) {
        if (!$this->init ||
            $forceInit) {
            $this->init();
        }
        if ($this->testMoteur &&
            !$forceTest) {
            return;
        }
        foreach($this->baseMoteurs as $cle => $valeur) {
            if (!isset($valeur["regexp"]) ||
                !is_array($valeur["regexp"])) {
                continue; /* foreach sur baseMoteur */
            }
            else {
                for ($i = 0; $i < sizeof($valeur["regexp"]); $i++) {
                    if (preg_match("#^".$valeur["regexp"][$i]."#i", $this->referant)) {
                        $this->estMoteur = true;
                        $this->nomMoteur = isset($valeur["nom"]) ? $valeur["nom"] : $cle;
                        $this->varianteMoteur = isset($valeur["variante"]) ? $valeur["variante"] : "";
                        $this->libelleMoteur = $this->nomMoteur." ".$this->varianteMoteur;
                        break; /* for sur les regexp */
                    }
                }
                if ($this->estMoteur) {
                    if (isset($valeur["query"]) &&
                        is_array($valeur["query"])) {
                        for ($i = 0; $i < sizeof($valeur["query"]); $i++) {
                            $sequenceQuery = explode(">", $valeur["query"][$i]);
                            $tmp = $this->requete;
                            for ($j = 0; $j < sizeof($sequenceQuery); $j++) {
                                parse_str($tmp);
                                $tmp = isset(${$sequenceQuery[$j]}) ? ${$sequenceQuery[$j]} : "";
                                if ($tmp == "") {
                                    break; /* for sur sequenceQuery */
                                }
                            }
                            if ($tmp != "") {
                                $this->requeteMoteur = trim(stripslashes(htmlentities($tmp)));
                                break; /* for sur query */
                            }
                        }
                    }
                    break; /* foreach sur baseMoteur */
                }
            }
        }
        $this->testMoteur = true;
        if (substr($this->requeteMoteur, 0, 6) == "cache:") {
            $position = strpos($this->requeteMoteur, " ");
            $this->requeteMoteur = substr($this->requeteMoteur, $position);
        }
    }
    
    private function init() {
        global $cfgIBDIR;
        clearstatcache();
        $cacheFile  = dirname(__FILE__)."/moteurs.dat";
        $moteursIni = dirname(__FILE__)."/moteurs.ini";
        if ($cfgIBDIR["useCache"]
            && file_exists($cacheFile)
            && filemtime($moteursIni) < filemtime($cacheFile)) {
            $this->baseMoteurs = unserialize(file_get_contents($cacheFile));
        }
        else {
            $this->baseMoteurs = parse_ini_file($moteursIni, true);
            if ($cfgIBDIR["useCache"]) {
                file_put_contents($cacheFile, serialize($this->baseMoteurs));
            }
        }
        $this->init = true;
    }
}
?>